/* filename:    filthome.g
** description: The program filters out the technology processes
** created:     09/27/01
*/

new;
closeall;
library homelib, pgraph, user;
cls;
#include c:\projects\active\homep\gauss\pathspec.g;

/******************************************************** 
** Import data on productivity growth, home investment growth,
** and hours: series (nobs,3)
** observations from 1965:I to 1997:IV
*/
#include c:\projects\active\homep\Gauss\loaddata.g

nobs  = rows(series);       /* number of observations          */
YY    = series[1:nobs,.];   /* Note that YY is defined defined over the entire series */
T0    = 20;

/* Open files with Parameter Draws
*/
lmodel = "m1";
lprior = "1";
mhrun  = "1";
mspec  = 2;

lpara = parapath $+ "\\mhrun" $+ mhrun $+ "\\" $+ lmodel  $+ lprior $+ "par";
open fhpara = ^lpara for read;

/* Define some Constants
*/
nsim   = 10;         /* block size */
skip   = 1;          /* skip = 1: use only first draw from block */
nburn  = 10001;

/* Initialize Output files
*/
opath = resupath $+ "\\mhrun" $+ mhrun $+ "\\";
oat = opath $+ lmodel $+ lprior $+ "at" ;
obt = opath $+ lmodel $+ lprior $+ "bt" ;
oxt = opath $+ lmodel $+ lprior $+ "xt" ;


create fhat = ^oat with AT, nobs-T0, 8;
create fhbt = ^obt with BT, nobs-T0, 8;
create fhxt = ^oxt with XT, nobs-T0, 8;

/* Initialization
*/ 
eofloop = 0;
loopct  = 0;
pararow = seekr(fhpara,nburn);

do until eofloop; 

   /* Read blocks of size nsim
   */
   parasim = readr( fhpara, nsim );

   nread = rows(parasim);
   
   if skip == 0;   
      Atsim = zeros(nread,nobs-T0);
      btsim = zeros(nread,nobs-T0);
      xtsim = zeros(nread,nobs-T0);
   else;
      Atsim = zeros(1,nobs-T0);
      btsim = zeros(1,nobs-T0);
      xtsim = zeros(1,nobs-T0);
   endif;

   j = 1;
   do until j > rows(parasim);

      /* For each element of the block, conduct the filtering
      */   

     statesim = dsgefilt( parasim[j,.]', mspec, YY);
   
     Atsim[j,.] = statesim[T0+1:nobs,1]';
     Btsim[j,.] = statesim[T0+1:nobs,2]';
     xtsim[j,.] = statesim[T0+1:nobs,3]'+ ( parasim[j,3]*seqa(1,1,nobs-T0) )';

     locate 1,1;
     "Iteration  " j "of" nsim;

     if skip == 1;
        j = rows(parasim)+2;
     else;
        j = j+1;
     endif;
  endo;

  /* Write Output
  */
  wr = writer(fhat,Atsim);
  wr = writer(fhbt,Btsim);
  wr = writer(fhxt,Xtsim);

  loopct = loopct + 1;
  locate 10,1;
  "Loop     " loopct;
  eofloop = eof(fhpara);

endo;

closeall fhat, fhbt, fhxt, fhpara;
