/* filename:    filtplot.g
** description: Compute posterior mean 
**              and plot filtered shock processes
*/

new;
closeall;
library user, homelib, pgraph;
cls;
outwidth 128;
#include c:\projects\active\homep\gauss\pathspec.g;

/******************************************************************
**         Load Parameter Draws from MH Output
*/
lmodel = "m1";
lprior = "1";
mhrun  = "1";
lpath  = resupath $+ "\\mhrun" $+ mhrun $+ "\\";

oat  = lpath $+ lmodel $+ lprior $+ "at" ;
obt  = lpath $+ lmodel $+ lprior $+ "bt" ;
oxt  = lpath $+ lmodel $+ lprior $+ "xt" ;

open fhat = ^oat for read;
open fhbt = ^obt for read;
open fhxt = ^oxt for read;

drawrow = readr( fhat, 1); 
drawdim = cols(drawrow);
drawrow = seekr( fhat, 1);


/* Part 1: Compute the mean of x(i)
*/
atmean   = zeros(1,drawdim);
btmean   = zeros(1,drawdim);
xtmean   = zeros(1,drawdim);

eofloop = 0;
nblock  = 5;
ndraws  = 0;

do until eofloop; 

   atblock = readr( fhat, nblock );
   btblock = readr( fhbt, nblock );
   xtblock = readr( fhxt, nblock );

   atmean  = atmean + sumc(atblock)';
   btmean  = btmean + sumc(btblock)';
   xtmean  = xtmean + sumc(xtblock)';

   ndraws = ndraws + rows(atblock);

   locate 1,1;
   "Part 1";
   "Draws " ndraws;

   eofloop = eof(fhat);

endo;

atmean   = atmean'/ndraws;
btmean   = btmean'/ndraws;
xtmean   = xtmean'/ndraws;

nobs     = rows(atmean);
ti       = seqa(1969,0.25,nobs);


/* Plot Impulse Response Functions
*/

graphset;
begwind;
margin(0,0,0.2,0.2);
window(3,1,0);
fonts("microb");

   _ptitlht = 0.3;
   _paxht = 0.3;
   _pnumht = 0.25;
   _pcolor =2 ;
   _pltype = {6 1 2 2};
   _protate = 1;
   _plctrl = 0;
   _plwidth = 5.5;
   _psymsiz = 5;

   xlabel("Year");
   xtics(1969,1998,1,1);

   ylow  = -0.06;
   yhigh =  0.06;
   ytics(ylow,yhigh,0.01,1);
   _pline = ( 1 ~ 1 ~ 1958.25 ~ ylow ~ 1958.25 ~ yhigh ~ 4 ~ 1 )|
            ( 1 ~ 1 ~ 1961 ~ ylow ~ 1961 ~ yhigh ~ 4 ~ 1 )|
            ( 1 ~ 1 ~ 1970.75 ~ ylow ~ 1970.75 ~ yhigh ~ 4 ~ 1 )|
            ( 1 ~ 1 ~ 1975 ~ ylow ~ 1975 ~ yhigh ~ 4 ~ 1 )|
            ( 1 ~ 1 ~ 1980.5 ~ ylow ~ 1980.5 ~ yhigh ~ 4 ~ 1 )|
            ( 1 ~ 1 ~ 1982.75 ~ ylow ~ 1982.75 ~ yhigh ~ 4 ~ 1 )|
            ( 1 ~ 1 ~ 1991 ~ ylow ~ 1991 ~ yhigh ~ 4 ~ 1 ) |
            ( 1 ~ 6 ~ 1957.5 ~ ylow ~ 1957.5 ~ yhigh ~ 4 ~ 1 ) |
            ( 1 ~ 6 ~ 1960.25 ~ ylow ~ 1960.25 ~ yhigh ~ 4 ~ 1 ) |
            ( 1 ~ 6 ~ 1969.75 ~ ylow ~ 1969.75 ~ yhigh ~ 4 ~ 1 ) |
            ( 1 ~ 6 ~ 1973.75 ~ ylow ~ 1973.75 ~ yhigh ~ 4 ~ 1 ) |
            ( 1 ~ 6 ~ 1980 ~ ylow ~ 1980 ~ yhigh ~ 4 ~ 1 ) |
            ( 1 ~ 6 ~ 1981.5 ~ ylow ~ 1981.5 ~ yhigh ~ 4 ~ 1 ) |
            ( 1 ~ 6 ~ 1990.5 ~ ylow ~ 1990.5 ~ yhigh ~ 4 ~ 1 ) ;

       
   title("Temporary Market Technology a(t)");
   xy(ti,atmean);
   nextwind;

   ylow  = -0.15;
   yhigh =  0.15;
   ytics(ylow,yhigh,0.02,1);
   _pline = ( 1 ~ 1 ~ 1958.25 ~ ylow ~ 1958.25 ~ yhigh ~ 4 ~ 1 )|
            ( 1 ~ 1 ~ 1961 ~ ylow ~ 1961 ~ yhigh ~ 4 ~ 1 )|
            ( 1 ~ 1 ~ 1970.75 ~ ylow ~ 1970.75 ~ yhigh ~ 4 ~ 1 )|
            ( 1 ~ 1 ~ 1975 ~ ylow ~ 1975 ~ yhigh ~ 4 ~ 1 )|
            ( 1 ~ 1 ~ 1980.5 ~ ylow ~ 1980.5 ~ yhigh ~ 4 ~ 1 )|
            ( 1 ~ 1 ~ 1982.75 ~ ylow ~ 1982.75 ~ yhigh ~ 4 ~ 1 )|
            ( 1 ~ 1 ~ 1991 ~ ylow ~ 1991 ~ yhigh ~ 4 ~ 1 ) |
            ( 1 ~ 6 ~ 1957.5 ~ ylow ~ 1957.5 ~ yhigh ~ 4 ~ 1 ) |
            ( 1 ~ 6 ~ 1960.25 ~ ylow ~ 1960.25 ~ yhigh ~ 4 ~ 1 ) |
            ( 1 ~ 6 ~ 1969.75 ~ ylow ~ 1969.75 ~ yhigh ~ 4 ~ 1 ) |
            ( 1 ~ 6 ~ 1973.75 ~ ylow ~ 1973.75 ~ yhigh ~ 4 ~ 1 ) |
            ( 1 ~ 6 ~ 1980 ~ ylow ~ 1980 ~ yhigh ~ 4 ~ 1 ) |
            ( 1 ~ 6 ~ 1981.5 ~ ylow ~ 1981.5 ~ yhigh ~ 4 ~ 1 ) |
            ( 1 ~ 6 ~ 1990.5 ~ ylow ~ 1990.5 ~ yhigh ~ 4 ~ 1 ) ;

   title("Temporary Home Technology b(t)");
   xy(ti,btmean);
   nextwind;


   ylow  = 0.00;
   yhigh = 0.65;
   ytics(ylow,yhigh,0.05,1);
   _pline = ( 1 ~ 1 ~ 1958.25 ~ ylow ~ 1958.25 ~ yhigh ~ 4 ~ 1 )|
            ( 1 ~ 1 ~ 1961 ~ ylow ~ 1961 ~ yhigh ~ 4 ~ 1 )|
            ( 1 ~ 1 ~ 1970.75 ~ ylow ~ 1970.75 ~ yhigh ~ 4 ~ 1 )|
            ( 1 ~ 1 ~ 1975 ~ ylow ~ 1975 ~ yhigh ~ 4 ~ 1 )|
            ( 1 ~ 1 ~ 1980.5 ~ ylow ~ 1980.5 ~ yhigh ~ 4 ~ 1 )|
            ( 1 ~ 1 ~ 1982.75 ~ ylow ~ 1982.75 ~ yhigh ~ 4 ~ 1 )|
            ( 1 ~ 1 ~ 1991 ~ ylow ~ 1991 ~ yhigh ~ 4 ~ 1 ) |
            ( 1 ~ 6 ~ 1957.5 ~ ylow ~ 1957.5 ~ yhigh ~ 4 ~ 1 ) |
            ( 1 ~ 6 ~ 1960.25 ~ ylow ~ 1960.25 ~ yhigh ~ 4 ~ 1 ) |
            ( 1 ~ 6 ~ 1969.75 ~ ylow ~ 1969.75 ~ yhigh ~ 4 ~ 1 ) |
            ( 1 ~ 6 ~ 1973.75 ~ ylow ~ 1973.75 ~ yhigh ~ 4 ~ 1 ) |
            ( 1 ~ 6 ~ 1980 ~ ylow ~ 1980 ~ yhigh ~ 4 ~ 1 ) |
            ( 1 ~ 6 ~ 1981.5 ~ ylow ~ 1981.5 ~ yhigh ~ 4 ~ 1 ) |
            ( 1 ~ 6 ~ 1990.5 ~ ylow ~ 1990.5 ~ yhigh ~ 4 ~ 1 ) ;

   title("Common Technology x(t)");
   xy(ti,xtmean);
   endwind;



