/* filename:    homepred.g
** description: The program converts the DSGE parameters
**              into impulse responses, etc
** created:     21/10/99
*/

/* new; */
closeall;
library homelib, pgraph, user;
cls;

#include c:\projects\active\homep\gauss\pathspec.g;

/* Open files with Parameter Draws
*/
lmodel = "m1";
lprior = "1";
mhrun  = "1";
mspec  = 2;

lpara = parapath $+ "\\mhrun" $+ mhrun $+ "\\" $+ lmodel  $+ lprior $+ "par";
open fhpara = ^lpara for read;

/* Define some Constants
*/
nsim   = 1000;         /* block size */
nirf   = 40;           /* number of impulse responses */
nomega = 50;           /* number of grid points for spectrum */
omegagr= seqa(0.0625, 0.33*pi/(nomega-1), nomega);


/* Initialize Output files
*/
opath = resupath $+ "\\mhrun" $+ mhrun $+ "\\";

oir11 = opath $+ lmodel $+ lprior $+ "r11" $+ mhrun;
oir12 = opath $+ lmodel $+ lprior $+ "r12" $+ mhrun;
oir13 = opath $+ lmodel $+ lprior $+ "r13" $+ mhrun;

oir21 = opath $+ lmodel $+ lprior $+ "r21" $+ mhrun;
oir22 = opath $+ lmodel $+ lprior $+ "r22" $+ mhrun;
oir23 = opath $+ lmodel $+ lprior $+ "r23" $+ mhrun;

oir31 = opath $+ lmodel $+ lprior $+ "r31" $+ mhrun;
oir32 = opath $+ lmodel $+ lprior $+ "r32" $+ mhrun;
oir33 = opath $+ lmodel $+ lprior $+ "r33" $+ mhrun;

ospy1 = opath $+ lmodel $+ lprior $+ "sy1" $+ mhrun;
ospy2 = opath $+ lmodel $+ lprior $+ "sy2" $+ mhrun;
ospy3 = opath $+ lmodel $+ lprior $+ "sy3" $+ mhrun;

ospl1 = opath $+ lmodel $+ lprior $+ "sl1" $+ mhrun;
ospl2 = opath $+ lmodel $+ lprior $+ "sl2" $+ mhrun;
ospl3 = opath $+ lmodel $+ lprior $+ "sl3" $+ mhrun;

ovard = opath $+ lmodel $+ lprior $+ "vd"  $+ mhrun;
ocorr = opath $+ lmodel $+ lprior $+ "cor" $+ mhrun;

create fhir11 = ^oir11 with IR11, nirf, 8;
create fhir12 = ^oir12 with IR12, nirf, 8;
create fhir13 = ^oir13 with IR13, nirf, 8;

create fhir21 = ^oir21 with IR21, nirf, 8;
create fhir22 = ^oir22 with IR22, nirf, 8;
create fhir23 = ^oir23 with IR23, nirf, 8;

create fhir31 = ^oir31 with IR31, nirf, 8;
create fhir32 = ^oir32 with IR32, nirf, 8;
create fhir33 = ^oir33 with IR33, nirf, 8;

create fhspy1 = ^ospy1 with SPY1, nomega, 8;
create fhspy2 = ^ospy2 with SPY2, nomega, 8;
create fhspy3 = ^ospy3 with SPY3, nomega, 8;

create fhspl1 = ^ospl1 with SPL1, nomega, 8;
create fhspl2 = ^ospl2 with SPL2, nomega, 8;
create fhspl3 = ^ospl3 with SPL3, nomega, 8;

create fhvard = ^ovard with VARD, 6, 8;
create fhcorr = ^ocorr with CORR, 1, 8;

/* Initialization
*/ 

eofloop = 0;
loopct  = 0;

do until eofloop; 

   /* Read blocks of size nsim
   */
   parasim = readr( fhpara, nsim );
   npara = cols(parasim);
   nread = rows(parasim);

   invalidsim = zeros(nread,1);
      
   Asim11 = zeros(nread,nirf);
   Asim12 = zeros(nread,nirf);
   Asim13 = zeros(nread,nirf);

   Asim21 = zeros(nread,nirf);
   Asim22 = zeros(nread,nirf);
   Asim23 = zeros(nread,nirf);

   Asim31 = zeros(nread,nirf);
   Asim32 = zeros(nread,nirf);
   Asim33 = zeros(nread,nirf);

   Ssimy1 = zeros(nread,nomega);
   Ssimy2 = zeros(nread,nomega);
   Ssimy3 = zeros(nread,nomega);

   Ssiml1 = zeros(nread,nomega);
   Ssiml2 = zeros(nread,nomega);
   Ssiml3 = zeros(nread,nomega);

   vardsim= zeros(nread,6);  
   corrsim= zeros(nread,1) ;

   j = 1;
   do until j > rows(parasim);

      /* For each element of the block, compute the desired statistic
      */

     {TTT, RRR, valid} = homesolv( parasim[j,.]', mspec);
     if valid > 0;
        invalidsim[j] = 1;
        goto eofparasim;
     endif;

     sigz = parasim[j,npara-2];
     siga = parasim[j,npara-1];
     sigb = parasim[j,npara];

     nstate = rows(TTT);

     /* create system matrices for state space model
     */

     /* composition of state vector
     ** dy(t) = 1, dy(t-1) = 2
     ** dlm(t)= 3, dlm(t-1) = 16
     ** dih(t)=12, dih(t-1) = 17
     */

     ZZ = zeros(3,nstate);
     ZZ[1,1] = 1; ZZ[1,3] = -1;  
     ZZ[2,12]= 1; 
     ZZ[3,3] = 1; 

     /* compute impulse responses for productivity, home investment, hours
     */

     IRFdz = zeros(3,nirf);
     IRFda = zeros(3,nirf); 
     IRFdb = zeros(3,nirf);
  
     /* Define initial state vector vector 
     */
     alpdz = zeros(nstate,1);
     alpda = zeros(nstate,1);
     alpdb = zeros(nstate,1);

     /* Shocks
     */
     u_dz   = sigz*ones(3,nirf).*(100|100|0) ;
     v_dz   = (100|0  |0)*sigz ~ zeros(3,nirf-1);

     u_da   = zeros(3,nirf);
     v_da   = (0 |100 |0)*siga ~ zeros(3,nirf-1);

     u_db   = zeros(3,nirf);
     v_db   = (0 |0 |100)*sigb ~ zeros(3,nirf-1);

     /* Initialize Loop variable t 
     */
     t=1;

     DO UNTIL t > nirf;

      /* update the vector of state variables alpha
      */
      alpdz = TTT*alpdz + RRR*v_dz[.,t];
      irfdz[.,t] = ZZ*alpdz + u_dz[.,t];

      alpda = TTT*alpda + RRR*v_da[.,t];
      irfda[.,t] = ZZ*alpda + u_da[.,t];

      alpdb = TTT*alpdb + RRR*v_db[.,t];
      irfdb[.,t] = ZZ*alpdb + u_db[.,t];

      t=t+1;

     ENDO;
     
     /* Variance Decomposition via impulse responses 
     */
/*
     vardy2 = ( irfdz[1,.]*irfdz[1,.]' / ( irfdz[1,.]*irfdz[1,.]' + irfda[1,.]*irfda[1,.]'  + irfdb[1,.]*irfdb[1,.]' ) )~
              ( irfda[1,.]*irfda[1,.]' / ( irfdz[1,.]*irfdz[1,.]' + irfda[1,.]*irfda[1,.]'  + irfdb[1,.]*irfdb[1,.]' ) )~
              ( irfdb[1,.]*irfdb[1,.]' / ( irfdz[1,.]*irfdz[1,.]' + irfda[1,.]*irfda[1,.]'  + irfdb[1,.]*irfdb[1,.]' ) );

     vardl2 = ( irfdz[3,.]*irfdz[3,.]' / ( irfdz[3,.]*irfdz[3,.]' + irfda[3,.]*irfda[3,.]'  + irfdb[3,.]*irfdb[3,.]' ) )~
              ( irfda[3,.]*irfda[3,.]' / ( irfdz[3,.]*irfdz[3,.]' + irfda[3,.]*irfda[3,.]'  + irfdb[3,.]*irfdb[3,.]' ) )~
              ( irfdb[3,.]*irfdb[3,.]' / ( irfdz[3,.]*irfdz[3,.]' + irfda[3,.]*irfda[3,.]'  + irfdb[3,.]*irfdb[3,.]' ) );
*/
     Asim11[j,.] = irfdz[1,1:nirf];
     Asim12[j,.] = irfda[1,1:nirf];
     Asim13[j,.] = irfdb[1,1:nirf];

     Asim21[j,.] = irfdz[2,1:nirf];
     Asim22[j,.] = irfda[2,1:nirf];
     Asim23[j,.] = irfdb[2,1:nirf];

     Asim31[j,.] = irfdz[3,1:nirf];
     Asim32[j,.] = irfda[3,1:nirf];
     Asim33[j,.] = irfdb[3,1:nirf];

     Corrsim[j,.] = ((1+(siga/sigz)^2)*(1+(sigb/sigz)^2))^(-0.5);

     /* Conduct Spectral Analysis
     */

     /* composition of state vector
     ** dy(t) = 1, dy(t-1) = 2
     ** dlm(t)= 3, dlm(t-1) = 16
     ** dih(t)=12, dih(t-1) = 17
     */

     ZZ = zeros(2,nstate);
     ZZ[1,1] = 1; ZZ[1,2] = -1;  
     ZZ[2,3] = 1; 
   
     vardl = zeros(1,3);
     vardy = zeros(1,3);

     {spec,gam} = spectr(TTT, RRR, ZZ, (sigz|0|0) );
     Ssimy1[j,.] = spec[1,.];
     Ssiml1[j,.] = spec[2,.];
     vardy[1,1] = gam[1,1];
     vardl[1,1] = gam[2,2];


     {spec,gam} = spectr(TTT, RRR, ZZ, (0|siga|0) );
     Ssimy2[j,.] = spec[1,.];
     Ssiml2[j,.] = spec[2,.];
     vardy[1,2] = gam[1,1];
     vardl[1,2] = gam[2,2];


     {spec,gam} = spectr(TTT, RRR, ZZ, (0|0|sigb) );
     Ssimy3[j,.] = spec[1,.];
     Ssiml3[j,.] = spec[2,.];
     vardy[1,3] = gam[1,1];
     vardl[1,3] = gam[2,2];

     vardy = vardy/sumc(vardy');
     vardl = vardl/sumc(vardl');
 


     vardsim[j,.] = vardy ~ vardl;

     eofparasim:

/*
     locate 1,1;
     "Iteration  " j "of" nsim;
     "Correlation" corrsim[j,.];
     "Variance D (Y) " vardy;
     "Variance D (L) " vardl;
*/

     j = j+1;
  endo;

  /* Delete Missing Entries
  */
  Asim11 = delif(Asim11, invalidsim);
  Asim12 = delif(Asim12, invalidsim);
  Asim13 = delif(Asim13, invalidsim);

  Asim21 = delif(Asim21, invalidsim);
  Asim22 = delif(Asim22, invalidsim);
  Asim23 = delif(Asim23, invalidsim);

  Asim31 = delif(Asim31, invalidsim);
  Asim32 = delif(Asim32, invalidsim);
  Asim33 = delif(Asim33, invalidsim);

  Ssimy1 = delif(Ssimy1, invalidsim);
  Ssimy2 = delif(Ssimy2, invalidsim);
  Ssimy3 = delif(Ssimy3, invalidsim);

  Ssiml1 = delif(Ssiml1, invalidsim);
  Ssiml2 = delif(Ssiml2, invalidsim);
  Ssiml3 = delif(Ssiml3, invalidsim);

  vardsim = delif(vardsim, invalidsim);
  corrsim = delif(corrsim, invalidsim);  

  /* Write Output
  */
  wr = writer(fhir11,Asim11);
  wr = writer(fhir12,Asim12);
  wr = writer(fhir13,Asim13);

  wr = writer(fhir21,Asim21);
  wr = writer(fhir22,Asim22);
  wr = writer(fhir23,Asim23);

  wr = writer(fhir31,Asim31);
  wr = writer(fhir32,Asim32);
  wr = writer(fhir33,Asim33);

  wr = writer(fhspy1,Ssimy1);
  wr = writer(fhspy2,Ssimy2);
  wr = writer(fhspy3,Ssimy3);

  wr = writer(fhspl1,Ssiml1);
  wr = writer(fhspl2,Ssiml2);
  wr = writer(fhspl3,Ssiml3);

  wr = writer(fhvard,vardsim);
  wr = writer(fhcorr,corrsim);

  loopct = loopct + 1;
  locate 10,1;
  "Loop     " loopct;
  eofloop = eof(fhpara);
  "Number of Invalid Draws" sumc(invalidsim);

endo;


closeall fhir11, fhir12, fhir13,
         fhir21, fhir22, fhir23,
         fhir31, fhir32, fhir33,
         fhspy1, fhspy2, fhspy3,
         fhspl1, fhspl2, fhspl3,
         fhpara, fhvard, fhcorr;

clear Asim11, Asim12, Asim13,
      Asim21, Asim22, Asim23,
      Asim31, Asim32, Asim33,
      Ssimy1, Ssimy2, Ssimy3,
      Ssiml1, Ssiml2, Ssiml3,
      vardsim, corrsim;


/* Procedures Start Here
*****************************/

proc (2) = spectr(TTT, RRR, ZZZ, sig);
/* Compute the Spectrum as a function of the
** system matrices
*/
local nstate, TT, RR, ZZ, sigma, spec, i, S1, Pt, GAM,
      p, d, q, TTsvd, RRsvd, Ptsvd;

nstate = rows(TTT);

/* Expand the state vector by epsz(t)
*/
TT = ( TTT ~ zeros(nstate,1) )|
     ( zeros(1,nstate) ~ 0   );

RR = RRR | (1 ~ 0 ~ 0);

ZZ = ZZZ ~ (1|0);

nstate = nstate + 1;

sigma = eye(3) .* (sig[1]^2 | sig[2]^2 | sig[3]^2);

spec = zeros(2,nomega);

i = 1;
do until i > nomega;
   
   S1 = complex( eye(nstate) - TT*cos(omegagr[i]),   TT*sin(omegagr[i]) );  
   spec[.,i] = real( diag( ZZ* (inv(S1)*(RR*sigma*RR')*inv(S1)'/(2*pi) )* ZZ' ) );

   i = i+1;

endo;

i = rank(TT);
{p, d, q} = svd1(TT);
q = q';
p = p[.,1:i];
q = q[1:i,.];
d = d[1:i,1:i];

TTsvd = q*p*d;
rrsvd = q*RR;

Ptsvd = reshape(inv(eye(i^2) - (TTsvd.*.TTsvd))*vec(RRsvd*SIGMA*RRsvd'),i,i);
Pt    = RR*sigma*RR' + p*d*Ptsvd*d*p'; 
GAM   = ZZ*Pt*ZZ';

/*
Pt = reshape(inv(eye(nstate^2) - (TT.*.TT))*vec(RR*SIGMA*RR'),nstate,nstate);
GAM = ZZ*Pt*ZZ';
*/

retp(spec,gam);
endp;
