/* filename:    homeprio.g
** description: The program generates draws from the prior of the 
**              home production model
** created:     09/18/01
*/

library pgraph, user, homelib;
cls;

block_n    = 110;
block_size = 1000;
mhrun      = "0";   /* NOTE: we use mhrun = 0 to denote prior */

/* Set path and import the information for the prior density
**
** Models: HOME2
** Priors: 1 
**
*/
#include c:\projects\active\homep\gauss\pathspec.g;

lmodel = "m1";
lprior = "1";
npara  = 18;
mspec  = 2;

priorfile = lmodel $+ lprior $+ "pri.out";
load path=^priopath prior[npara,3] = ^priorfile;

pmean  = prior[.,1];
pstdd  = prior[.,2];
pshape = prior[.,3];


/* Initialize Output files
*/
opath = parapath $+ "\\mhrun" $+ mhrun $+ "\\";
opara = opath  $+ lmodel $+ lprior $+ "par";
create fhpara=^opara with PARAM, npara, 8;

/* Loop to generate parameter draws
*/
block_it = 1;
para_acpt = 0;

do until block_it > block_n;

   parasim = zeros(block_size,npara);
   i = 1; 
   do until i > npara;
   
      if pshape[i] == 1; /* BETA Prior */
         a = (1-pmean[i])*pmean[i]^2/pstdd[i]^2 - pmean[i];
         b = a*(1/pmean[i] - 1);
         parasim[.,i] = rndbeta(block_size,1,a,b);
      
      elseif pshape[i] == 2; /* GAMMA PRIOR */
         b = pstdd[i]^2/pmean[i];
         a = pmean[i]/b;
         parasim[.,i] = b*rndgam(block_size,1,a);
      
      elseif pshape[i] == 3; /* GAUSSIAN PRIOR */
         a = pmean[i];
         b = pstdd[i];
         parasim[.,i] = a + b*rndn(block_size,1);

      elseif pshape[i] == 4; /* INVGAMMA PRIOR */
         a = pmean[i];
         b = pstdd[i];
         parasim[.,i] = sqrt( b*a^2/sumc( (rndn(b,block_size))^2 ) );
      
      elseif pshape[i] == 0; /* No Prior - Fixed */
         a = pmean[i];
         parasim[.,i] = a;

      endif;

      i = i+1;
   endo; 
   
   parasim = delif(parasim, ((parasim[.,6] .== 0) .OR ( parasim[.,7] .== 0) ));
   para_acpt = para_acpt + rows(parasim);
      
   locate 1,1;
   "Processing Block" block_it;
   
   wr = writer(fhpara,parasim);
   block_it = block_it + 1;

endo;

"";
"Accepted" para_acpt "out of" block_n*block_size;

closeall fhpara;
