/* filename:    homermse.g
** description: The program computes the RMSE at the posterior mean
**              
** created:     10/01/01
** modified:   
*/

new;
closeall;
library user, pgraph, homelib;
cls;

/******************************************************** 
**      Models: 1 = RBC
**              2 = HOME Production
*/ 
#include c:\projects\active\homep\Gauss\pathspec.g
lmodel = "m1";
mspec  = 2;
lprior = "1";
mhrun  = "1";
npara  = 18;

/******************************************************** 
** Import data on productivity growth, home investment growth,
** and hours: series (nobs,3)
** observations from 1955:I to 1997:IV
*/
#include c:\projects\active\homep\Gauss\loaddata.g

nobs  = rows(series);       /* number of observations          */
YY    = series[1:nobs,.];   /* Note that YY is defined defined over the entire series */
T0    = 20;

/******************************************************** 
** Load Posterior mean parameter estimates
*/
lpath       = parapath $+ "\\mhrun" $+ mhrun $+ "\\";
lpara       = lpath $+ lmodel $+ lprior $+ "par" $+ "s";
open fhpara = ^lpara for read;

load path   = ^priopath para_names;
para        = readr(fhpara,1)';
closeall fhpara;

"Parameter   | Estimate ";
outmat = para_names'~para;
let mask[1,2] = 0 1;
let fmt[2,3] =
   "-*.*s " 10 4
   "*.*lf " 10 4;
d = printfm(outmat,(0 ~ 1),fmt);

/* Compute RMSE
*/
{lnpy, obsmean, obsvar } = evalhome( para, mspec, T0 ,YY);

prederror = YY - obsmean;
prederror = prederror[T0+1:rows(prederror),.];

"";
"Number of Observations used to compute RMSE";
rows(prederror);
"";
"RMSE" sqrt(meanc(prederror^2));
