/***************************************************************/
/*                                                             */
/*            Convert Spectrum: VAR Decomposition in Simplex   */
/*                          Coordinates                        */
/*                       Frank Schorfheide                     */
/*                                                             */
/***************************************************************/

/* Filename:  spctconv.g
** created:   09/20/01
*/

new;
closeall;
library pgraph;
cls;

#include c:\projects\active\homep\gauss\pathspec.g;

nread   = 10000;
skip    = 0;

nomega = 50;          /* number of grid points for spectrum */
omegagr= seqa(0.0625, 0.33*pi/(nomega-1), nomega);
spectr = 2-2*cos(omegagr);
spectr = spectr';

mhrun_all  = "0" $| "1";
lprior     = "1";
lmodel_all = "m0" $| "m1";
depvar_all = "y"  $| "l";

modind = 1;
do until modind > 2;
   mhind = 2;
   do until mhind > 2;
      varind = 1;
      do until varind > 2;

         mhrun = mhrun_all[mhind];
         lmodel = lmodel_all[modind];
         depvar = depvar_all[varind];

         lpath = resupath $+ "\\mhrun" $+ mhrun $+"\\";
         opath = resupath $+ "\\mhrun" $+ mhrun $+"\\";

         locate 1,1;
         "Model   " lmodel;
         "MhRun   " mhrun;
         "Dep Var " depvar;

         gosub transform;

         varind = varind+1;
      endo;
      mhind = mhind + 1;
   endo;     	         
   modind = modind + 1;
endo; 

end;

/*****************************************************************
**  S U B R O U T I N E
*/
transform:

/* Open Input File
*/
osp11 = lpath $+ lmodel $+ lprior $+ "s" $+ depvar $+ "1" $+mhrun;
osp12 = lpath $+ lmodel $+ lprior $+ "s" $+ depvar $+ "2" $+mhrun;
osp13 = lpath $+ lmodel $+ lprior $+ "s" $+ depvar $+ "3" $+mhrun;

open fhsp11 = ^osp11 for read;
open fhsp12 = ^osp12 for read;
open fhsp13 = ^osp13 for read;

specrow = readr(fhsp11,1);
nspec   = cols(specrow);
specrow = seekr(fhsp11,1);

/* Initialize Output files
*/

/* Variance Decomposition
*/
ospd1 = lpath $+ lmodel $+ lprior $+ "sd" $+ depvar $+ "1" $+mhrun;
ospd2 = lpath $+ lmodel $+ lprior $+ "sd" $+ depvar $+ "2" $+mhrun;
ospd3 = lpath $+ lmodel $+ lprior $+ "sd" $+ depvar $+ "3" $+mhrun;
create fhspd1 = ^ospd1 with SPD1, 1, 8;
create fhspd2 = ^ospd2 with SPD2, 1, 8;
create fhspd3 = ^ospd3 with SPD3, 1, 8;


/* Variance Decomposition in Simplex Coordinates
*/
ospds = opath $+ lmodel $+ lprior $+ "sd" $+ depvar $+ "s" $+mhrun;
create fhspds = ^ospds with SPDS, 2, 8;

eofloop = 0;
ndraws  = 0;
loopct  = 0;

do until eofloop; 

   /* Compute Decomposition
   */
  
   sp11sim = readr( fhsp11, nread);
   sp12sim = readr( fhsp12, nread);
   sp13sim = readr( fhsp13, nread);

   sp11sim = sumc((sp11sim./spectr)');
   sp12sim = sumc((sp12sim./spectr)');
   sp13sim = sumc((sp13sim./spectr)');

   specsim = sp11sim+sp12sim+sp13sim;
   spd1sim = sp11sim./specsim;
   spd2sim = sp12sim./specsim;
   spd3sim = sp13sim./specsim;

   wr = writer(fhspd1,spd1sim);
   wr = writer(fhspd2,spd2sim);
   wr = writer(fhspd3,spd3sim);

   spdssim = zeros(rows(sp11sim),2);

   /* Convert coordinates
   */
   ysim = sqrt(3/2)*spd3sim;
   xsim = ( sqrt(2)*spd2sim ./ (spd2sim+spd1sim) - 1/sqrt(2) ) .*(sqrt(3/2)-ysim)/sqrt(3/2);

   j = 1;
   do until j > 1;
      spdssim[.,1+2*(j-1):2*j] = xsim[.,j] ~ ysim[.,j];
      j = j+1;
   endo;
     
   if skip == 1;
      wr = writer(fhspds,spdssim[1,.]);
   else;
      wr = writer(fhspds,spdssim);
   endif;

   l = 2*(sqrt(3/2)-ysim[1,1])*tan(pi/6);

   /* Check ratios
   */
   /*   
   locate 1,1;
   "Xsim                " xsim[1,1];
   "Ysim                " ysim[1,1];
   "";
   height_b = ysim[1,1];
   area_b = 0.5*height_b*sqrt(2);

   length = sqrt( ysim[1,1]^2 + (1/sqrt(2)-xsim[1,1])^2);
   angle  = arctan(ysim[1,1]/( 1/sqrt(2)-xsim[1,1] ));
   angle2 = pi/3 - angle;
   height_z = length*sin(angle2);
   area_z = 0.5*height_z*sqrt(2);

   area_total = 0.5*sqrt(2)*sqrt(2)*sin(pi/3);

   area_a = area_total - area_z - area_b;

   area_a = area_a/area_total;
   area_b = area_b/area_total;
   area_z = area_z/area_total;

   "Area Decomposition  " area_z area_a area_b;
   "Spectral Decomp     " spd1sim[1,1] spd2sim[1,1] spd3sim[1,1];

   "";
   "Home Shock          " spd3sim[1,1];
   "Projection Ratio    " ysim[1,1]/sqrt(3/2);
   "";
   "Ratio of Tech Shocks" spd2sim[1,1]./spd1sim[1,1];
   "Projection Ratio    " (l/2+xsim[1,1])/(l/2-xsim[1,1]);

   k = keyw;   
   */   
   loopct = loopct + 1;
   locate 10,1;
   "Iteration" loopct;

   eofloop = eof(fhsp11);

endo;

closeall fhsp11, fhsp12, fhsp13, 
         fhspd1, fhspd2, fhspd3,
         fhspds;

return;
