/* filename:    spctpmom.g
** description: Compute posterior mean and std and CI based on
**              output of posterior simulator
*/

new;
closeall;
library user, homelib, pgraph;
cls;
outwidth 128;

/******************************************************************
**         Load Parameter Draws from MH Output
*/
#include c:\projects\active\homep\gauss\pathspec.g;

nburn       = 10001;        /* Number of initial draws to be discarded */
hpdprob     = 0.90 ;


lmodel = "m0" $|
         "m1" ;
mhrun  = "0" $| "1";
lprior = "1";
spctid = "sdy1" $| "sdy2" $| "sdy3" $|
         "sdl1" $| "sdl2" $| "sdl3" $| "vd";


modind = 1;
do until modind > 2;
   mhind = 2;
   do until mhind > 2;
      spctind = 1;
      do until spctind > 7;
      
         lpath  = resupath $+ "\\mhrun" $+ mhrun[mhind] $+ "\\";
         opath  = resupath $+ "\\mhrun" $+ mhrun[mhind] $+ "\\";

         ldraw  = lpath $+ lmodel[modind] $+ lprior $+ spctid[spctind] $+ mhrun[mhind];
         odraws = opath $+ lmodel[modind] $+ lprior $+ spctid[spctind] $+ mhrun[mhind] $+ "s";
        
         gosub computmom;
         
         spctind = spctind + 1;
      endo;
      mhind = mhind + 1;
   endo;
   modind = modind+1;
endo;

end;
         
/***********************************************************
**  S U B R O U T I N E
*/
computmom:

open fhdraw = ^ldraw for read;

drawrow = readr( fhdraw, nburn); 
drawdim = cols(drawrow);
drawrow = seekr( fhdraw, nburn);

/* Part 1: Compute the mean of x(i) and x^2(i)
*/
drawmean   = zeros(1,drawdim);
drawsqmean = zeros(1,drawdim);
drawcross  = zeros(drawdim,drawdim);

eofloop = 0;
nblock  = 50000;
ndraws  = 0;

do until eofloop; 

   drawblock = readr( fhdraw, nblock );

   drawmean  = drawmean    + sumc(drawblock)';
   drawsqmean = drawsqmean + sumc(drawblock^2)';
   drawcross  = drawcross  + drawblock'*drawblock;  

   ndraws = ndraws + rows(drawblock);

   eofloop = eof(fhdraw);

endo;

drawmean   = drawmean/ndraws;
drawsqmean = drawsqmean/ndraws;
drawstdd   = sqrt(drawsqmean - (drawmean)^2);
drawcov    = drawcross/ndraws - drawmean'*drawmean;


/* Part 2: HPD Interval
*/
drawrow     = seekr( fhdraw, nburn); 
drawci      = zeros(2,drawdim);

j = 1;
do until j > drawdim;

   /* Read only the j'th column
   */
   drawrow   = seekr( fhdraw, nburn); 
   drawblock = readr( fhdraw, 1);

   drawcol = drawblock[1,j];
   eofloop = 0;
   ndraws  = 1;
   
   do until eofloop; 

      drawblock = readr( fhdraw, nblock );
      drawcol   = drawcol | drawblock[.,j];
      ndraws    = ndraws + rows(drawblock);
      eofloop   = eof(fhdraw);
         
   endo;
   
   drawci[.,j] = hpdint(drawcol,hpdprob);
   j = j+1;

endo;
clear drawcol;


/* Report Posterior Mean and stderror
**
*/
"MODEL                " lmodel[modind];
"PRIOR                " lprior;
"MH-Run               " mhrun[mhind];
"Spectrum Component   " spctid[spctind];

"---------------------------------------------";
" " ;

"Mean / S.E.";
drawmean'~drawstdd';
"C.I.";
drawci';

"=============================================";
"";
"";

create fhdraws = ^odraws with DRAWSUM, drawdim, 8;
writer(fhdraws, real(drawmean | drawstdd | drawcov | drawci) );
closeall fhdraws; 

return;

/**************************************************************/
/*                         Procedures                         */
/**************************************************************/

proc(1) = hpdint(draws,percent);
local     ndraws, drawdim, hpdband, nwidth, i, drawcoli,
          bup, minwidth, newwidth, j;

drawdim   = cols(draws);
ndraws    = rows(draws);
hpdband   = zeros(2,drawdim);
nwidth    = int(percent*ndraws);

i = 1;
do until i > drawdim;
   drawcoli = draws[.,i];
   /* sort response for period i, element 1 is max
   */
   drawcoli = rev(sortc(drawcoli,1));
   bup   = 1;
   minwidth  = drawcoli[1] - drawcoli[nwidth];
   j = 2;
   do until j > (ndraws-nwidth+1);
      newwidth = drawcoli[j] - drawcoli[j+nwidth-1];
      if newwidth < minwidth;
         bup = j;
         minwidth = newwidth;
      endif;
      j = j+1;
   endo;
   hpdband[2,i] = drawcoli[bup];
   hpdband[1,i] = drawcoli[bup+nwidth-1];
   i = i+1;
endo;
retp(hpdband);
endp;
