/***************************************************************/
/*                                                             */
/*     VEC analysis: sampling from the posterior,              */
/*                                                             */
/*                       Frank Schorfheide                     */
/*                                                             */
/***************************************************************/

/* Filename:  vecgibbs.g
** created:   09/21/01
*/

/* new; */
closeall;
library pgraph;

#include c:\projects\active\homep\Gauss\pathspec.g
#include c:\projects\active\homep\Gauss\vecspec.g

/* Configuration
*/
lprior = "1";
lmodel = "m0";
mhrun  = "1";

/* Gibbs Sampler Setup
**
*/
nsim     = 1000;
nblocks  = 110;
la_scale = 0.0;


/* Import data on productivity growth, homeinvestment growth, and hours
**        observations from 1965:I to 1997:IV
*/
#include c:\projects\active\homep\Gauss\loaddata.g

nobs     = rows(series);       /* number of observations          */
nvar     = cols(series);
npara    = nvar*(1+ttrend+1+p*nvar)+ 0.5*nvar*(nvar+1) + 1; 
YY       = series[1+p:nobs,.];
XXvec    = seriesvec[1+p:nobs,.];


if ttrend == 0;
   k     = 1 + nvar*p;            /* number of coefficients          */
   XX    = ones(nobs-p, k);
   if p > 0;
   i = 1;
   do until i > p;
     XX[.,2+(i-1)*nvar:1+i*nvar] = series[p-i+1:nobs-i,.];
     i = i+1;
   endo;
   endif;
else;
   k     = 2 + nvar*p;            /* number of coefficients          */
   XX    = ones(nobs-p, k);
   XX[.,2] = seqa(1,1,nobs-p);
   if p > 0;
   i = 1;
   do until i > p;
     XX[.,3+(i-1)*nvar:2+i*nvar] = series[p-i+1:nobs-i,.];
     i = i+1;
   endo;
   endif;
endif;

nobs    = nobs - p;
T0      = T0 - p;

/* Initialize Output files
*/
opath = parapath $+ "\\mhrun" $+ mhrun $+ "\\";
opara = opath  $+ lmodel $+ lprior $+ "par";
ostat = opath  $+ lmodel $+ lprior $+ "sta";

statname = { POST LIKE };

create fhstat = ^ostat with ^statname, 2, 8;
create fhpara = ^opara with PARAM, npara, 8;

cls;


/*************************************************************
**           Posterior Simulator
*/

/* Initialize the Gibbs Sampler
*/
la_sim  = 1 + la_scale*rndn(1,1);


XXs     = (XXvec*(1|-la_sim) )~XX;
CC_sim  = inv(XXs'*XXs)*XXs'*YY;
CCt_sim = CC_sim';
Sig_sim = (YY-XXs*CC_sim)'*(YY-XXs*CC_sim)/nobs;

iblock = 1;
do until iblock > nblocks; 

  parasim   = zeros(nsim,npara);
  likesim   = zeros(nsim,1);
  postsim   = zeros(nsim,1);

  j = 1;
  do until j > nsim;

     /* Draw from lambda conditional on CC,Sigma
     */
     YYtrans = YY - XX*CC_sim[2:rows(CC_sim),.] - XXvec[.,1]*CC_sim[1,.];
     XXtrans = -XXvec[.,2]*CC_sim[1,.];
     vhat_la = 1/sumc(diag(inv(Sig_sim)*(XXtrans'*XXtrans)));
     mhat_la = vhat_la*sumc(diag(inv(Sig_sim)*(XXtrans'*YYtrans)));
     vT_la   = 1/(1/vhat_la + 1/v0_la);
     mT_la   = vT_la*(mhat_la/vhat_la + m0_la/v0_la);
     la_sim  = mT_la + sqrt(vT_la)*rndn(1,1);

     /* Draw from Sigma, CC conditional on lambda
     */
     XXs     = (XXvec*(1|-la_sim) )~XX;
     CC_hat  = inv(XXs'*XXs)*XXs'*YY;    
     H       = (YY-XXs*CC_hat)'*(YY-XXs*CC_hat);
     Z       = chol(inv(H))'*rndn(nvar,nobs-k-1);
     Sig_sim = inv(Z*Z');

     VC     = Sig_sim.*.inv(XXs'*XXs);
     C_sim  = vec(CC_hat) + chol(VC)'*rndn(nvar*(k+1),1);
     CCt_sim= reshape(C_sim,nvar,k+1);
     CC_sim = CCt_sim';

     parasim[j,.] = la_sim ~ Sig_sim[1,1:3] ~ Sig_sim[2,2:3] ~ Sig_sim[3,3] ~ C_sim'; 
     
     /* Evaluate likelihood function
     */
     likesim[j] = -(nvar/2)*((nobs-T0))*ln(2*pi) - ((nobs-T0)/2)*ln(det(Sig_sim)) 
                 - 0.5*sumc(diag(  inv(Sig_sim)*( (YY[T0+1:nobs,.]-XXs[T0+1:nobs,.]*CCt_sim')'
                            *(YY[T0+1:nobs,.]-XXs[T0+1:nobs,.]*CCt_sim'))));

     /* Evaluate the prior at parameter draws
     */
     CCprior  = inv(XXs[1:T0,.]'*XXs[1:T0,.])*XXs[1:T0,.]'*YY[1:T0,.];
     HHprior  = (YY[1:T0,.]-XXs[1:T0,.]*CCprior)'*(YY[1:T0,.]-XXs[1:T0,.]*CCprior);
     nup      = T0-k-1;

     lnprior = -(k+1)*ln(2*pi)-0.5*ln(det(Sig_sim.*.inv(XXs[1:T0,.]'*XXs[1:T0,.]))) 
               -0.5*sumc(diag( inv(Sig_sim)*(CCt_sim' - CCprior)'*(XXs[1:T0,.]'*XXs[1:T0,.])*(CCt_sim'-CCprior) ))
               -( 0.5*nup*nvar*ln(2) + 0.25*nvar*(nvar-1)*ln(pi) + sumc( ln(gamma( seqa(nup,-1,nvar)/2)) )  )
               +(nup/2)*ln(det(HHprior)) - ((nup+nvar+1)/2)*ln(det(Sig_sim)) - 0.5*sumc(diag( inv(Sig_sim)*HHprior ))
               -0.5*ln(2*pi) - 0.5*ln(v0_la) -0.5*(la_sim - m0_la)'*inv(v0_la)*(la_sim - m0_la) ;

     postsim[j] = likesim[j] + lnprior;
         
     /* Screen output
     */
     /*
     locate 1,1;
     "Block          " iblock "out of" nblocks;
     "Simulation step" j "out of" nsim;

     "Cointvector    " la_sim;
     "Likelihood     " likesim[j]     ;
     "Prior          " lnprior        ;
     */    
     j = j+1;

  endo;
 
  /* Save the results
  */
  wr = writer(fhstat,postsim~likesim);
  wr = writer(fhpara,parasim);

  /* Screen output
  */
  locate 1,1;
  "Block          " iblock "out of" nblocks;
  
  iblock = iblock+1;
endo;

closeall fhpara, fhstat;


